/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package application;

import globals.Constants;
import globals.Globals;
import globals.SettingsManager;

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Hashtable;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerListModel;

import core.GameSwitcher;
import core.GameUndoManager;

/**
 * The dialog which manages user preferences.
 */
class PreferencesDialog extends ApplicationDialog implements ActionListener {
	
	// Lazy eight symbol
	private static final String infinity_symbol = "\u221e";
	
	JCheckBox[] gridsettings;
	JSlider propsliderX;
	JSlider propsliderY;
	JCheckBox antiabox, querybox;
	JSpinner undo_spinner;
	GameSwitcher game_panel;

	PreferencesDialog(GameSwitcher gp) {
		super(Globals.graphics_frame, Globals.text
				.getString("preferences_title"), true);
		JTabbedPane tabs;
		JComponent panel;
		
		// We need to store the game panel so that we can ask it to repaint
		// itself.
		game_panel = gp;

		JPanel contentPane = new JPanel();
		setContentPane(contentPane);
		contentPane.setOpaque(true);
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));

		// TABBED PANE

		tabs = new JTabbedPane();

		contentPane.add(tabs);

		// GENERAL PANEL
		
        panel = new JPanel();
        panel.setLayout(new BoxLayout(panel,BoxLayout.PAGE_AXIS));
        
		querybox = new JCheckBox(Globals.text.getString("query_for_quits"),
				SettingsManager.app_queryForQuits);
		panel.add(querybox);

		// UNDO SPINNER
		
		// Create a list of strings corresponding to the available undo choices
		String[] temp = new String[GameUndoManager.availableUndoLevels.length];
		for (int i = 0; i < GameUndoManager.availableUndoLevels.length; i++)
			temp[i] = toNumUndos(GameUndoManager.availableUndoLevels[i]);
		
		undo_spinner = new JSpinner(new SpinnerListModel(temp));
        undo_spinner.setValue(toNumUndos(SettingsManager.app_maxUndos));
		JLabel undo_label = new JLabel(" "+Globals.text.getString("undo_spinner_label"));

		// FIXME this renders horribly.
		JComponent box1 = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JComponent box = new JPanel(new GridLayout(1,2));
		box.add(undo_spinner);
		box.add(undo_label);
		box1.add(box);
		panel.add(box1);

		tabs.addTab(Globals.text.getString("general_tab"), panel);
		tabs.setMnemonicAt(0, KeyEvent.VK_1);

		// GRAPHICS PANEL
		panel = new Box(BoxLayout.Y_AXIS);

		antiabox = new JCheckBox(Globals.text.getString("graphics_antiA"),
				SettingsManager.graphics_antialiasing);
		panel.add(antiabox);

		// Graphics proportion slider
		Hashtable<Integer,JLabel> h = new Hashtable<Integer,JLabel>();
		h.put(new Integer(Constants.text_dialog_graphicsPropMinPoint),
				new JLabel(Globals.text.getString("graphics_propMin")));
		h.put(new Integer(Constants.text_dialog_graphicsPropMaxPoint),
				new JLabel(Globals.text.getString("graphics_propMax")));

		panel.add(new JLabel(Globals.text.getString("graphics_Xproportion")));
		propsliderX = new JSlider(0, 100,
				(int) ((1 - SettingsManager.graphics_Xproportion) * 100));
		// to give it useful labels, we need to do the following:
		propsliderX.setLabelTable(h);
		propsliderX.setPaintLabels(true);
		panel.add(propsliderX);

		panel.add(new JLabel(Globals.text.getString("graphics_Yproportion")));
		propsliderY = new JSlider(0, 100,
				(int) ((1 - SettingsManager.graphics_Yproportion) * 100));
		// to give it useful labels, we need to do the following:
		propsliderY.setLabelTable(h);
		propsliderY.setPaintLabels(true);
		panel.add(propsliderY);

		tabs.addTab(Globals.text.getString("graphics_tab"), panel);
		tabs.setMnemonicAt(1, KeyEvent.VK_2);

		// BUTTONS
		JPanel buttons = new JPanel();

		JButton defaults_button = new JButton(Globals.text
				.getString("defaults_button"));
		defaults_button.addActionListener(this);
		buttons.add(defaults_button);

		JButton okay_button = new JButton(Globals.text.getString("okay_button"));
		okay_button.addActionListener(this);
		buttons.add(okay_button);

		JButton cancel_button = new JButton(Globals.text
				.getString("cancel_button"));
		cancel_button.addActionListener(this);
		buttons.add(cancel_button);

		contentPane.add(buttons);

		pack();
		setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command == Globals.text.getString("defaults_button")) {
			if (Dialogs.confirmRestore(this)) {
				SettingsManager.restoreSettings();
				restore();
				game_panel.update();
			}
		} else if (command == Globals.text.getString("okay_button")) {
			apply();
			setVisible(false);
			game_panel.update(); // just in case we've altered the view
			Dialogs.close_preferencesDialog();
		} else if (command == Globals.text.getString("cancel_button"))
			cancel();
	}
	
	void cancel() {
		setVisible(false);
		restore();
	}
	
	/**
	 * Convert from an int to a string suitable for the undo spinner.
	 */
	private String toNumUndos (int i) {
		//FIXME This should be more defensive by only return an allowed value
		if (i >= 0)
			return Integer.toString(i);
		else
			return infinity_symbol;
	}
	
	/**
	 * Parse the contents of the undo spinner.
	 */
	private int parseNumUndos (String s) {
		if (s == infinity_symbol) {
			return GameUndoManager.unbounded;
		} else
			return Integer.parseInt(s);
	}
	
	private void apply() {
		SettingsManager.graphics_Xproportion = 1 - (propsliderX.getValue() / 100.0);
		SettingsManager.graphics_Yproportion = 1 - (propsliderY.getValue() / 100.0);
		SettingsManager.graphics_antialiasing = antiabox.getSelectedObjects() != null;
		// Settings.file_reloadLevelsOnReset = reloadbox
		// .getSelectedObjects() != null;
		SettingsManager.app_queryForQuits = querybox.getSelectedObjects() != null;
		SettingsManager.app_maxUndos = parseNumUndos((String) undo_spinner.getValue());
	}

	private void restore() {
		propsliderX.setValue((int) ((1 - SettingsManager.graphics_Xproportion) * 100));
		propsliderY.setValue((int) ((1 - SettingsManager.graphics_Yproportion) * 100));
		antiabox.setSelected(SettingsManager.graphics_antialiasing);
		// reloadbox.setSelected(Settings.file_reloadLevelsOnReset);
		querybox.setSelected(SettingsManager.app_queryForQuits);
		undo_spinner.setValue(toNumUndos(SettingsManager.app_maxUndos));
	}
}
