/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package sokoban;

import java.util.LinkedList;

import level.FloorGrid;
import level.Level;
import level.UnitShapeGroup;
import tessellation.Position;
import tessellation.TessellationManager;
import core.GameInfo;
import file.LevelParseException;
import file.LevelParser;
import globals.Globals;

/**
 * Class which parses .xsb (XSokoban) files.
 * 
 * The other Oban parsers subclass this, and several (Trioban and Octoban) are
 * sensitive to the column in which the level starts, so whitespace on the left
 * cannot be ignored.
 */
abstract class ObanLevelParser extends LevelParser {
	protected String getGameTitle() {
		return "Sokoban";
	}

	protected Level parseLevel(String t, String a, String[] sa, int levelNo, int levelDepth, GameInfo ts)
			throws LevelParseException {
		TessellationManager.set_tessellation(ts.get_tessellation());
		LinkedList<Position> blocks = new LinkedList<Position>();
		LinkedList<Position> goals = new LinkedList<Position>();
		LinkedList<Position> people = new LinkedList<Position>();
		LinkedList<Position> floorTiles = new LinkedList<Position>();
		UnitShapeGroup walls;
		int i, j;

		if (sa[0] == null)
			throw new Error("First string array element was null");


                int maxLineLen = 0;
		for (i = 0; i < sa.length; i++) {
			if (sa[i] == null)
				throw new Error("String array element " + i + " was null");
                        else if (Solver.DEBUG) System.out.println(sa[i]);
			if (sa[i].length() > maxLineLen) maxLineLen = sa[i].length();
		}

		for (i = 0; i < sa.length; i++)
			for (j = 0; j < sa[i].length(); j++) {
				if (!XsbParseUnit.charIsUnit(sa[i].charAt(j)))
					throw new LevelParseException(
							Globals.text.getString("error_xsb_invalidCharacter"), i,
							sa.length - i);
				if (XsbParseUnit.charIsBlockUnit(sa[i].charAt(j)))
					blocks.add(TessellationManager.newPosition(j, i));
				if (XsbParseUnit.charIsGoalUnit(sa[i].charAt(j)))
					goals.add(TessellationManager.newPosition(j, i));
				if (XsbParseUnit.charIsPersonUnit(sa[i].charAt(j))) {
					people.add(TessellationManager.newPosition(j, i));
				}
			}

		if (people.size() == 0)
			throw new LevelParseException(
					Globals.text.getString("error_xsb_noStartPosition"),
					sa.length - 1, 1);
		if (goals.size() != blocks.size())
			throw new LevelParseException(
					Globals.text.getString("error_xsb_blockGoalMismatch"),
					sa.length - 1, 1);
		walls = parseWallsFromStringArray(sa);
		for (Position p : people) {
			blankTilesFromPos(sa,p,floorTiles);
			//floorTiles = parseBlankFloorFromStringArray(sa, people.getFirst());
		}
                if (Solver.DEBUG && (levelNo>0)) {
                    System.out.println(
                        "Sorry, solving only the first level in DEBUG mode.");
                } else {
                    new Solver(sa, maxLineLen, levelDepth);
                }
		
		return new ObanLevel(t, a, people, blocks, goals, walls, new FloorGrid(floorTiles), ts);
	}
}
