/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package tessellation;

import java.awt.geom.Point2D;

/**
 * A class which encapsulates the notion of position.
 */
public abstract class Position {
	protected int x, y;

	public boolean equals(Object other) {
		if (other instanceof Position)
			return ((x == ((Position) other).x) && (y == ((Position) other).y));
		else
			return false;
	}

	/**
	 * Returns a x-index into an array for storing positions
	 */
	public int array_x() {
		return x;
	}

	/**
	 * Returns a y-index into an array for storing positions
	 */
	public int array_y() {
		return y;
	}

	/**
	 * Return a position which can be used as the bound to an array to hold both
	 * this and the provided position
	 */
	public Position containingArrayBound(Position other) {
		return TessellationManager.newPosition((x >= other.x) ? x : other.x,
				(y >= other.y) ? y : other.y);
	}

	/**
	 * Does this position have a edge in the given direction.
	 */
	public boolean hasEdge(Direction dir) {
		return true;
	}

	/**
	 * Does this position have a neighbour in the given direction.
	 * 
	 * The default behaviour is to use hasEdge.
	 */
	public boolean hasNeighbour(Direction dir) {
		return hasEdge(dir);
	}

	/**
	 * start point of an inner edge
	 */
	public Point2D startPointOfInnerEdge(Direction dir) {
		Point2D previous_corner = startPointOfEdge(dir.prevEdgeAt(this));
		Point2D next_corner = startPointOfEdge(dir.nextEdgeAt(this));
		Point2D this_corner = startPointOfEdge(dir);
		double innermost_x = (previous_corner.getX() + next_corner.getX()) / 2;
		double innermost_y = (previous_corner.getY() + next_corner.getY()) / 2;
		double inner_weighting = .2;
		return new Point2D.Double((innermost_x * inner_weighting)
				+ this_corner.getX() * (1 - inner_weighting),
				(innermost_y * inner_weighting) + this_corner.getY()
						* (1 - inner_weighting));
	}

	public Point2D endPointOfInnerEdge(Direction dir) {
		return startPointOfInnerEdge(dir.nextEdgeAt(this));
	}

	/**
	 * Constructs the position which neighbours this position in the provided
	 * direction
	 */
	public abstract Position neighbour(Direction dir);

	/**
	 * Returns a position which corresponds to adding the provided positions
	 * together.
	 */
	public abstract Position plus(Position rel);

	/**
	 * Calculates the a position which is the relative difference between two
	 * positions
	 */
	public abstract Position minus(Position rel);

	/**
	 * Returns the position which extends furthest up the screen
	 */
	public abstract Position furthestUp(Position other);

	/**
	 * Returns the position which extends furthest right on the screen
	 */
	public abstract Position furthestRight(Position other);

	/**
	 * Returns the position which extends furthest down on the screen
	 */
	public abstract Position furthestDown(Position other);

	/**
	 * Returns the position which extends furthest left on the screen
	 */
	public abstract Position furthestLeft(Position other);

	/**
	 * returns the coordinates for the start of the given edge of the current
	 * position
	 */
	public abstract Point2D startPointOfEdge(Direction dir);

	/**
	 * returns the coordinates of the upperlefthand corner which bounds the
	 * given position
	 */
	public abstract Point2D topLeftCornerPoint();

	/**
	 * returns the coordinates of the lowerrighthand corner which bounds the
	 * given position
	 */
	public abstract Point2D bottomRightCornerPoint();

	/**
	 * returns the coordinates of the upperlefthand corner of the unit's inner
	 * circle
	 */
	public abstract Point2D innerCircleTopLeftCornerPoint();

	/**
	 * returns the coordinates of the lowerrighthand corner of the unit's inner
	 * circle
	 */
	public abstract Point2D innerCircleBottomRightCornerPoint();

	/**
	 * returns the coordinates of the upperlefthand corner of the unit's inner
	 * circle
	 */
	public abstract Point2D innerSquareTopLeftCornerPoint();

	/**
	 * returns the coordinates of the lowerrighthand corner of the unit's inner
	 * circle
	 */
	public abstract Point2D innerSquareBottomRightCornerPoint();

	public String toString() {
		return "Pos(" + x + "," + y + ")";
	}

	/**
	 * Some values and methods which are useful for subclasses
	 */
	final static double root2 = StrictMath.sqrt(2.0);
	final static double root3 = StrictMath.sqrt(3.0);
	final static double tworoot3 = 2.0 * root3;
	final static double root2th = 1.0 / StrictMath.sqrt(2.0);

	static boolean isEven(int i) {
		return i % 2 == 0;
	}

	static boolean isOdd(int i) {
		return !isEven(i);
	}
}