/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package tessellation;

import globals.Settings;

import java.awt.geom.Point2D;

import application.KeySettingsDialog;
import application.KeySettingsPanel;

import core.PlayerAction;

/**
 * A static factory class which builds Directions and Positions of the
 * appropriate type.
 * 
 * It would be nicer to have levels parametrised on Tessellations. Unforunately,
 * since the type parameters would be lost at run-time, the levels could not create
 * positions or directions of the appropriate class. Thus, every component of a level
 * would have to have a reference to a TessellationFactory. As an alternative, I
 * use a static TessellationManager, which stores the "current" Tessellation. Alas,
 * this means only one type of tessellation can be available in the system at the same
 * time.
 */
public class TessellationManager {
	// I have to set this to a default, since levels are parsed before I choose
	// which
	// 
	private static Tessellation tessellation = Tessellation.SQUARE;

	/**
	 * Set the current tessellation.
	 */
	public static void set_tessellation(Tessellation t) {
		tessellation = t;
	}

	/**
	 * Get an index corresponding to the current tessellation.
	 */
	public static int get_tessellationIndex() {
		return tessellation.ordinal();
	}

	/**
	 * Get the number of tessellations
	 */
	public static int get_numberOfTessellations() {
		return Tessellation.values().length;
	}

	static final TessellationFactory[] the_factories = { new SquareFactory() };

	public static Direction newDirection() {
		return the_factories[get_tessellationIndex()].newDirection();
	}

	public static Direction newDirection(int startDir) {
		return the_factories[get_tessellationIndex()].newDirection(startDir);
	}

	public static Position newPosition() {
		return the_factories[get_tessellationIndex()].newPosition();
	}

	public static Position newPosition(int array_x, int array_y) {
		return the_factories[get_tessellationIndex()].newPosition(array_x,
				array_y);
	}

	public static Position newPosition(Position pos) {
		return the_factories[get_tessellationIndex()].newPosition(pos);
	}

	public static Position newPosition(Point2D p) {
		return the_factories[get_tessellationIndex()].newPosition(p);
	}

	/**
	 * Attempts to match a key with a direction key for a given player
	 */
	public static PlayerAction getPlayerActionFromKey(int k) {
		return the_factories[get_tessellationIndex()]
				.getPlayerDirectionFromKey(k);
	}


	/**
	 * Returns a panel which can be used to edit the keys
	 */
	public static KeySettingsPanel getKeySettingsPanel(KeySettingsDialog ksd,
			int i) {
		return the_factories[i].newKeySettingsPanel(ksd);
	}

	public static void restoreDefaults() {
		for (Tessellation t : Tessellation.values())
			the_factories[t.ordinal()].restoreDefaults();
	}

	public static void accessKeySettings() {
		for (Tessellation t : Tessellation.values())
			the_factories[t.ordinal()].accessKeySettings();
	}

	public static void getSettings(Settings scb) {
		for (Tessellation t : Tessellation.values())
			the_factories[t.ordinal()].getSettings(scb);
	}

	public static void updateKeySettings() {
		for (TessellationFactory f : the_factories)
			f.updateKeySettings();
	}
}
